@@ -11,7 +11,7 @@ class Argon2:
         self._ph = None
 
         if app is not None:
-            self.init_app(app)
+            self.init_app(app)  # pragma: no cover
 
     def init_app(self, app):
         """
@@ -20,19 +20,12 @@ class Argon2:
         """
         app.config.setdefault("ARGON2_ENCODING", "utf-8")
         app.config.setdefault("ARGON2_TIME_COST", _argon2.DEFAULT_TIME_COST)
-        app.config.setdefault("ARGON2_HASH_LENGTH", _argon2.DEFAULT_HASH_LENGTH)
+        app.config.setdefault("ARGON2_HASH_LEN", _argon2.DEFAULT_HASH_LENGTH)
         app.config.setdefault("ARGON2_MEMORY_COST", _argon2.DEFAULT_MEMORY_COST)
         app.config.setdefault("ARGON2_PARALLELISM", _argon2.DEFAULT_PARALLELISM)
-        app.config.setdefault("ARGON2_RANDOM_SALT_LENGTH", _argon2.DEFAULT_RANDOM_SALT_LENGTH)
+        app.config.setdefault("ARGON2_SALT_LEN", _argon2.DEFAULT_RANDOM_SALT_LENGTH)
 
-        self._ph = _argon2.PasswordHasher(
-            encoding=app.config['ARGON2_ENCODING'],
-            time_cost=app.config['ARGON2_TIME_COST'],
-            hash_len=app.config['ARGON2_HASH_LENGTH'],
-            memory_cost=app.config['ARGON2_MEMORY_COST'],
-            parallelism=app.config['ARGON2_PARALLELISM'],
-            salt_len=app.config['ARGON2_RANDOM_SALT_LENGTH']
-        )
+        self._ph = _argon2.PasswordHasher(**app.config.get_namespace('ARGON2_'))
 
         if not hasattr(app, 'extensions'):
             app.extensions = dict()  # pragma: no cover
@@ -44,8 +37,6 @@ class Argon2:
         :param password:
         :return:
         """
-        if not password:
-            raise ValueError('password must be not empty')
         return self._ph.hash(password)
 
     def verify_hash(self, pw_hash, password, exc=False):
@@ -56,10 +47,12 @@ class Argon2:
         :param exc:
         :return:
         """
-        if exc is False:
-            try:
-                return self._ph.verify(pw_hash, password)
-            except (Argon2Error, InvalidHash):
-                return False
-        else:
+        try:
             return self._ph.verify(pw_hash, password)
+        except (Argon2Error, InvalidHash):
+            if exc is True:
+                raise  # pragma: no cover
+            return False
+
+
+argon2 = Argon2()
