@@ -1,7 +1,8 @@
 import flask
+from flask import current_app as cap
 from flask_response_builder import encoders
 
-from . import cap, httpcode
+from .http import httpcode
 from .utils.datastruct import ObjectDict
 
 
@@ -27,12 +28,27 @@ class Request(flask.Request):
         if payload is None:
             if not allow_empty:
                 flask.abort(httpcode.BAD_REQUEST, 'No JSON in request')
-            payload = ObjectDict()
+            payload = ObjectDict()  # pragma: no cover
 
         return ObjectDict.normalize(payload)
 
 
 class Response(flask.Response):
+    @staticmethod
+    def no_content(status=httpcode.NO_CONTENT, headers=None):
+        """
+
+        :param status:
+        :param headers:
+        :return:
+        """
+        response = flask.make_response(bytes())
+        response.headers.update(headers or {})
+        response.headers.pop('Content-Type', None)
+        response.headers.pop('Content-Length', None)
+        response.status_code = status
+        return response
+
     @staticmethod
     def send_file(directory, filename, **kwargs):
         """
@@ -52,12 +68,14 @@ class Response(flask.Response):
             return  # only to prevent warning # pragma: no cover
 
         # following headers works with nginx compatible proxy
-        if cap.use_x_sendfile is True and cap.config.get('ENABLE_ACCEL') is True:
+        if cap.use_x_sendfile is True and cap.config.ENABLE_ACCEL is True:
             resp.headers['X-Accel-Redirect'] = file_path
-            resp.headers['X-Accel-Charset'] = cap.config['ACCEL_CHARSET']
-            resp.headers['X-Accel-Limit-Rate'] = cap.config['ACCEL_LIMIT_RATE']
-            resp.headers['X-Accel-Expires'] = cap.config['SEND_FILE_MAX_AGE_DEFAULT']
-            resp.headers['X-Accel-Buffering'] = 'yes' if cap.config['ACCEL_BUFFERING'] else 'no'
+            resp.headers['X-Accel-Charset'] = cap.config.ACCEL_CHARSET or 'utf-8'
+            resp.headers['X-Accel-Buffering'] = 'yes' if cap.config.ACCEL_BUFFERING else 'no'
+            if cap.config.ACCEL_LIMIT_RATE:
+                resp.headers['X-Accel-Limit-Rate'] = cap.config.ACCEL_LIMIT_RATE
+            if cap.config.SEND_FILE_MAX_AGE_DEFAULT:
+                resp.headers['X-Accel-Expires'] = cap.config.SEND_FILE_MAX_AGE_DEFAULT
 
         return resp
 
@@ -69,3 +87,7 @@ class Flaskel(flask.Flask):
     request_class = Request
     response_class = Response
     json_encoder = encoders.JsonEncoder
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.version = None
