@@ -1,13 +1,17 @@
 import flask
 from requests import auth, exceptions as http_exc, request as send_request
 
-from flaskel import cap
+from flaskel.flaskel import cap
 from flaskel.utils.datastruct import ObjectDict
-from flaskel.utils.faker import FakeLogger
+from flaskel.utils.faker.logger import FakeLogger
 from flaskel.utils.uuid import get_uuid
-from . import http_status as httpcode
+from . import httpcode
 from .httpdumper import FlaskelHTTPDumper, HTTPDumper
 
+HTTPStatusError = (http_exc.HTTPError,)
+NetworkError = (http_exc.ConnectionError, http_exc.Timeout)
+all_errors = (*HTTPStatusError, *NetworkError)
+
 
 class HTTPTokenAuth(auth.AuthBase):
     def __init__(self, token):
@@ -23,7 +27,7 @@ class HTTPTokenAuth(auth.AuthBase):
         :param other:
         :return:
         """
-        return self.token == getattr(other, 'token', None)
+        return self.token == getattr(other, 'token', None)  # pragma: no cover
 
     def __ne__(self, other):
         """
@@ -31,7 +35,7 @@ class HTTPTokenAuth(auth.AuthBase):
         :param other:
         :return:
         """
-        return not self == other
+        return not self == other  # pragma: no cover
 
     def __call__(self, r):
         """
@@ -44,17 +48,27 @@ class HTTPTokenAuth(auth.AuthBase):
 
 
 class HTTPBase(HTTPDumper):
-    def __init__(self, dump_body=False, raise_on_exc=False, logger=None):
+    def __init__(self, dump_body=False, timeout=10, raise_on_exc=False, logger=None):
         """
 
         :param dump_body:
+        :param timeout:
         :param raise_on_exc:
         :param logger:
         """
-        self._dump_body = dump_body
+        self._dump_body = self._normalize_dump_flag(dump_body)
+        self._timeout = timeout
         self._raise_on_exc = raise_on_exc
         self._logger = logger or FakeLogger()
 
+    @staticmethod
+    def _normalize_dump_flag(dump_body):
+        if type(dump_body) is bool:
+            return dump_body, dump_body
+        if not dump_body:
+            return False, False
+        return dump_body
+
     def request(self, uri, dump_body=None, **kwargs):
         """
 
@@ -91,12 +105,26 @@ class HTTPClient(HTTPBase):
         if self._token:
             return HTTPTokenAuth(self._token)
 
-    def request(self, uri, method='GET',
+    def normalize_url(self, url):
+        if url.startswith('http'):
+            return url
+
+        return f"{self._endpoint}/{url.lstrip('/')}"
+
+    @staticmethod
+    def prepare_response(body=None, status=httpcode.SUCCESS, headers=None, exception=None):
+        return ObjectDict(
+            body=body or {}, status=status,
+            headers=headers or {}, exception=exception
+        )
+
+    def request(self, uri, method='GET', raise_on_exc=False,
                 dump_body=None, chunk_size=None, decode_unicode=False, **kwargs):
         """
 
-        :param method:
         :param uri:
+        :param method:
+        :param raise_on_exc:
         :param dump_body:
         :param chunk_size:
         :param decode_unicode:
@@ -106,33 +134,35 @@ class HTTPClient(HTTPBase):
         kwargs['auth'] = self.get_auth()
         if dump_body is None:
             dump_body = self._dump_body
-        if kwargs.get('stream') is True:
-            dump_body = False
+        else:
+            dump_body = self._normalize_dump_flag(dump_body)
+        if kwargs.get('stream') is True:  # if stream not dump response body
+            dump_body = (dump_body[0], False)
 
         try:
-            response = send_request(method, f"{self._endpoint}{uri}", **kwargs)
-            self._logger.info(self.dump_request(response.request))
-        except (http_exc.ConnectionError, http_exc.Timeout) as exc:
+            kwargs.setdefault('timeout', self._timeout)
+            url = self.normalize_url(uri)
+            req = ObjectDict(method=method, url=url, **kwargs)
+            self._logger.info(self.dump_request(req, dump_body[0]))
+            response = send_request(method, self.normalize_url(uri), **kwargs)
+        except NetworkError as exc:
             self._logger.exception(exc)
-            if self._raise_on_exc:
+            if raise_on_exc or self._raise_on_exc:
                 raise  # pragma: no cover
 
-            return ObjectDict(dict(
-                body={},
-                status=httpcode.SERVICE_UNAVAILABLE,
-                headers={},
-                exception=exc
-            ))
+            return self.prepare_response(
+                status=httpcode.SERVICE_UNAVAILABLE, exception=exc
+            )
 
+        log_resp = self.dump_response(response, dump_body[1])
         try:
             response.raise_for_status()
-        except http_exc.HTTPError as exc:
-            self._logger.warning(self.dump_response(response, dump_body))
+            self._logger.info(log_resp)
+        except HTTPStatusError as exc:
+            self._logger.warning(log_resp)
             response = exc.response
-            if self._raise_on_exc:
+            if raise_on_exc or self._raise_on_exc:
                 raise
-        else:
-            self._logger.info(self.dump_response(response, dump_body))
 
         if kwargs.get('stream') is True:
             body = response.iter_content(chunk_size, decode_unicode)
@@ -141,11 +171,11 @@ class HTTPClient(HTTPBase):
         else:
             body = response.text
 
-        return ObjectDict(dict(
+        return self.prepare_response(
             body=body,
             status=response.status_code,
             headers=dict(response.headers)
-        ))
+        )
 
     def get(self, uri, **kwargs):
         return self.request(uri, **kwargs)
@@ -162,6 +192,12 @@ class HTTPClient(HTTPBase):
     def delete(self, uri, **kwargs):
         return self.request(uri, method='DELETE', **kwargs)
 
+    def options(self, uri, **kwargs):
+        return self.request(uri, method='OPTIONS', **kwargs)
+
+    def head(self, uri, **kwargs):
+        return self.request(uri, method='HEAD', **kwargs)
+
 
 class JsonRPCClient(HTTPClient):
     def __init__(self, endpoint, uri, version='2.0', **kwargs):
@@ -213,9 +249,6 @@ class JsonRPCClient(HTTPClient):
         :param kwargs:
         :return:
         """
-        if not method:
-            raise ValueError('method must be a valid string')
-
         resp = super().request(
             self._uri,
             method='POST',
@@ -228,13 +261,34 @@ class JsonRPCClient(HTTPClient):
             **kwargs
         )
 
-        return ObjectDict(resp['body'] or {})
+        return resp.body or ObjectDict()
 
 
-class FlaskelHttp(HTTPClient, FlaskelHTTPDumper):
+class FlaskelHttp(FlaskelHTTPDumper, HTTPClient):
+    def __init__(self, endpoint, **kwargs):
+        kwargs.setdefault('logger', cap.logger)
+        super().__init__(endpoint, **kwargs)
+        self._timeout = cap.config.HTTP_TIMEOUT or self._timeout
+
     def request(self, uri, **kwargs):
         if flask.request.id:
-            kwargs.setdefault('headers', {})
+            if not kwargs.get('headers'):
+                kwargs['headers'] = {}
+            kwargs['headers'][cap.config.REQUEST_ID_HEADER] = flask.request.id
+
+        return super().request(uri, **kwargs)
+
+
+class FlaskelJsonRPC(FlaskelHTTPDumper, JsonRPCClient):
+    def __init__(self, endpoint, uri, **kwargs):
+        kwargs.setdefault('logger', cap.logger)
+        super().__init__(endpoint, uri, **kwargs)
+        self._timeout = cap.config.HTTP_TIMEOUT or self._timeout
+
+    def request(self, uri, **kwargs):
+        if flask.request.id:
+            if not kwargs.get('headers'):
+                kwargs['headers'] = {}
             kwargs['headers'][cap.config.REQUEST_ID_HEADER] = flask.request.id
 
         return super().request(uri, **kwargs)
