@@ -1,16 +1,7 @@
-from flask import current_app as cap
+from flask import current_app as cap, json
 
 
 class HTTPDumper:
-    @staticmethod
-    def padding(text):
-        """
-
-        :param text:
-        :return:
-        """
-        return "\n{}".format(text)
-
     @classmethod
     def dump_headers(cls, hdr, only=()):
         """
@@ -22,10 +13,12 @@ class HTTPDumper:
                 k1: v1
                 k2: v2
         """
+        hdr = hdr or {}
         if only:
             hdr = {k: hdr[k] for k in only if k in hdr}
 
-        return cls.padding('\n'.join('{}: {}'.format(k, v) for k, v in hdr.items()))
+        headers = '\n\t'.join(f"{k}: {v}" for k, v in hdr.items())
+        return headers.strip()
 
     @classmethod
     def response_filename(cls, headers):
@@ -36,6 +29,7 @@ class HTTPDumper:
         :param headers: http headers dict
         :return: only the file name
         """
+        headers = headers or {}
         hdr = headers.get('Content-Disposition')
         if not hdr:
             return None  # pragma: no cover
@@ -50,101 +44,69 @@ class HTTPDumper:
             return None
 
     @classmethod
-    def dump_request(cls, req, dump_body=None):
+    def dump_request(cls, req, dump_body=None, only_hdr=()):
         """
         dump http request: useful for logging
 
         :param req: Request instance
         :param dump_body: flag to enable or disable dump of request's body (overrides default)
+        :param only_hdr: dump only a subset of headers
         :return: prettified representation of input as string
         """
+        body = ''
         if dump_body is True:
-            body = req.body or "empty body"
-            if type(body) is not str:
-                body = body.decode()
-        else:
-            body = "request body not dumped"
+            body = getattr(req, 'json', None)
+            if body:
+                body = json.dumps(body)
+            else:
+                body = getattr(req, 'body', getattr(req, 'data', None))
+                if isinstance(body, (bytes, bytearray)):
+                    body = 'Binary body not dumped'
+            body = f"\nbody:\n{body}" if body else ''
 
-        headers = cls.dump_headers(req.headers)
-        return f"{req.method} {req.url}\nheaders:\n{headers}\nbody:\n{body}"
+        headers = cls.dump_headers(req.headers, only=only_hdr)
+        headers = f"\nheaders:\n\t{headers}" if headers else ''
+        return f"requested {req.method} {req.url}{headers}{body}"
 
     @classmethod
-    def dump_response(cls, response, dump_body=None):
+    def dump_response(cls, resp, dump_body=None, only_hdr=()):
         """
         dump http response: useful for logging
 
-        :param response: Response instance
+        :param resp: Response instance
         :param dump_body: flag to enable or disable dump of response's body (overrides default)
+        :param only_hdr: dump only a subset of headers
         :return: prettified representation of input as string
         """
-        resp = response
-        hdr = resp.headers
-
+        body = ''
         if dump_body is True:
             body = cls.response_filename(resp.headers) or resp.text
-            if type(body) is not str:
-                body = body.decode()
-        else:
-            body = "response body not dumped"
+            body = f"\nbody:\n{body}" if body else ''
 
         try:
             seconds = resp.elapsed.total_seconds()
         except AttributeError:  # because aiohttp.ClientResponse has not elapsed attribute
             seconds = 'N/A'
 
-        headers = cls.dump_headers(hdr)
-        status = resp.status_code if hasattr(resp, 'status_code') else resp.status
-        return f"time : {seconds} - status code: {status}\nheaders:\n{headers}\nbody:\n{body}"
+        headers = cls.dump_headers(resp.headers, only=only_hdr)
+        headers = f"\nheaders:\n\t{headers}" if headers else ''
+        try:
+            status = resp.status_code
+        except AttributeError:
+            status = getattr(resp, 'status', None)
+
+        return f"response time: {seconds} - status code: {status}{headers}{body}"
 
 
-class FlaskelHTTPDumper(HTTPDumper):
+class FlaskelHTTPDumper:
     @classmethod
     def dump_request(cls, req, dump_body=None):
-        """
-        dump http request: useful for logging
-
-        :param req: Request instance
-        :param dump_body: flag to enable or disable dump of request's body (overrides default)
-        :return: prettified representation of input as string
-        """
-        headers = ''
-        body = "request body not dumped"
-        hdr = cap.config.LOG_REQ_HEADERS
-
-        if hdr or dump_body:
-            headers = cls.dump_headers(req.headers, hdr)
-
-        if dump_body is True:
-            body = "empty body"
-            if type(req.body) is str:
-                body = req.body
-
-        return f"{req.method} {req.url}\nheaders:\n{headers}\nbody:\n{body}"
+        return HTTPDumper.dump_request(
+            req, dump_body, only_hdr=cap.config.LOG_REQ_HEADERS
+        )
 
     @classmethod
-    def dump_response(cls, response, dump_body=None):
-        """
-        dump http response: useful for logging
-
-        :param response: Response instance
-        :param dump_body: flag to enable or disable dump of response's body (overrides default)
-        :return: prettified representation of input as string
-        """
-        resp = response
-        hdr = cap.config.LOG_RESP_HEADERS or resp.headers
-
-        if dump_body is True:
-            body = cls.response_filename(resp.headers) or resp.text
-            if type(body) is not str:
-                body = body.decode()
-        else:
-            body = "response body not dumped"
-
-        try:
-            seconds = resp.elapsed.total_seconds()
-        except AttributeError:  # because aiohttp.ClientResponse has not elapsed attribute
-            seconds = 'N/A'
-
-        headers = cls.dump_headers(hdr)
-        status = resp.status_code if hasattr(resp, 'status_code') else resp.status
-        return f"time : {seconds} - status code: {status}\nheaders:\n{headers}\nbody:\n{body}"
+    def dump_response(cls, resp, dump_body=None):
+        return HTTPDumper.dump_response(
+            resp, dump_body, only_hdr=cap.config.LOG_RESP_HEADERS
+        )
