@@ -3,10 +3,11 @@ import os
 import click
 import decouple
 
-from . import AppBuilder
-from .utils.misc import parse_value
-from .utils.yaml import load_yaml_file, setup_yaml_parser
-from .wsgi import BaseApplication, WSGIFactory
+from .builder import AppBuilder
+from .utils import misc, yaml
+from .utils.datastruct import ObjectDict
+from .wsgi.base import BaseApplication
+from .wsgi.factory import WSGIFactory
 
 
 # noinspection PyUnusedLocal
@@ -21,7 +22,7 @@ def option_as_dict(ctx, param, value):
     ret = {}
     for opt in value:
         k, v = opt.split("=", 2)
-        ret.update({k: parse_value(v)})
+        ret.update({k: misc.parse_value(v)})
     return ret
 
 
@@ -60,6 +61,13 @@ class Server:
         metavar="KEY=VAL",
         help='wsgi configuration'
     )
+    option_app_config_attr = dict(
+        default={},
+        multiple=True,
+        callback=option_as_dict,
+        metavar="KEY=VAL",
+        help='app configuration'
+    )
 
     def __init__(self, app_factory=AppBuilder(), wsgi_factory=WSGIFactory()):
         """
@@ -74,12 +82,6 @@ class Server:
         assert isinstance(self._wsgi_factory, WSGIFactory)
 
     def _register_options(self, func):
-        """
-
-        :param func:
-        :return:
-        """
-
         @click.command()
         @click.option('-b', '--bind', **self.opt_bing_attr)
         @click.option('-d', '--debug', **self.opt_debug_attr)
@@ -87,6 +89,7 @@ class Server:
         @click.option('-l', '--log-config', **self.opt_log_config_attr)
         @click.option('-w', '--wsgi-server', **self.opt_wsgi_server_attr)
         @click.option('-D', '--wsgi-config', **self.option_wsgi_config_attr)
+        @click.option('-W', '--app-config', **self.option_app_config_attr)
         def _options(*args, **kwargs):
             return func(*args, **kwargs)
 
@@ -94,53 +97,47 @@ class Server:
 
     def run_from_cli(self, **kwargs):
         @self._register_options
-        def _forever(config, log_config, bind, debug, wsgi_server, wsgi_config):
-            self.serve_forever(
-                bind=bind,
-                debug=debug,
-                config=config,
-                log_config=log_config,
-                wsgi_server=wsgi_server,
-                wsgi_config=wsgi_config,
-                **kwargs
-            )
+        def forever(**params):
+            self.serve_forever(**params, **kwargs)
 
-        return _forever()
+        forever()
 
     @staticmethod
     def _prepare_config(filename, debug=None, bind=None, log_config=None):
         """
 
         :param filename:
-        :return: config
+        :param debug:
+        :param bind:
+        :param log_config:
+        :return:
         """
         default_env = 'development' if debug else 'production'
         env = decouple.config('FLASK_ENV', default=default_env)
 
         if filename is not None:
-            setup_yaml_parser()
-            config = load_yaml_file(filename)
-            if not config['app']['FLASK_ENV']:
-                config['app']['FLASK_ENV'] = env
+            yaml.setup_yaml_parser()
+            config = yaml.load_yaml_file(filename)
+            if not config.app.FLASK_ENV:
+                config.app.FLASK_ENV = env
         else:
-            config = dict(
+            config = ObjectDict(
                 app={'DEBUG': debug, 'FLASK_ENV': env},
                 wsgi={'bind': bind, 'debug': debug}
             )
 
-        os.environ['FLASK_ENV'] = config['app']['FLASK_ENV']
-
         if log_config is not None:
-            config['app']['LOG_FILE_CONF'] = log_config
+            config.app.LOG_FILE_CONF = log_config
 
         # debug flag enabled overrides config file
         if debug is True:
-            config['app']['DEBUG'] = True
+            config.app.DEBUG = True
 
+        os.environ['FLASK_ENV'] = config.app.FLASK_ENV
         return config
 
     def serve_forever(self, config=None, log_config=None, bind=None, debug=None,
-                      wsgi_class=None, wsgi_server=None, wsgi_config=None):
+                      wsgi_class=None, wsgi_server=None, wsgi_config=None, app_config=None):
         """
 
         :param config: app and wsgi configuration file
@@ -148,12 +145,14 @@ class Server:
         :param bind: address to bind
         :param debug: enable debug mode
         :param wsgi_class: optional a custom subclass of BaseApplication
-        :param wsgi_server: wsgi server chose
+        :param wsgi_server: wsgi server chosen
         :param wsgi_config: wsgi configuration
+        :param app_config: app configuration
         :return: never returns
         """
-        config = self._prepare_config(config, debug=debug, bind=bind, log_config=log_config)
-        wsgi_config = {**(wsgi_config or {}), **(config.get('wsgi') or {})}
+        config = self._prepare_config(config, debug, bind, log_config)
+        wsgi_config = {**(config.get('wsgi') or {}), **(wsgi_config or {})}
+        app_config = {**(config.get('app') or {}), **(app_config or {})}
 
         if wsgi_server:
             wsgi_class = self._wsgi_factory.get_class(wsgi_server)
@@ -162,6 +161,6 @@ class Server:
 
         assert issubclass(wsgi_class, BaseApplication)
 
-        app = self._app_factory.get_or_create(config.get('app'))
+        app = self._app_factory.get_or_create(app_config)
         wsgi = wsgi_class(app, options=wsgi_config)
         wsgi.run()  # run forever
