@@ -15,19 +15,19 @@ class FakeLogger:
         pass
 
 
-# noinspection PyMethodMayBeStatic,PyUnusedLocal
+# noinspection PyMethodMayBeStatic
 class DummyLogger:
     def debug(self, *args, **kwargs):
-        print(*args)
+        print("DEBUG:", *args)
 
     def info(self, *args, **kwargs):
-        print(*args)
+        print("INFO:", *args)
 
     def warning(self, *args, **kwargs):
-        print(*args)
+        print("WARNING:", *args)
 
     def error(self, *args, **kwargs):
-        print(*args)
+        print("ERROR:", *args)
 
     def exception(self, exc, *args, **kwargs):
-        print(*args)
+        print("CRITICAL:", *args)
